@@ -2,6 +2,8 @@
 #Housekeeping functions
 #################
 
+
+
 ###
 #Determine if a vector consists entirely of zeroes and ones
 ###
@@ -18,6 +20,7 @@ is.binary <- function(x) {
 #  return(sum(unique(x) == c(1,0)) + sum(unique(x) == c(0,1)) ==2)
 #}
 
+
 ###
 #Parses out response, treatment and covariates from formula
 #arguments:
@@ -25,38 +28,55 @@ is.binary <- function(x) {
 #data: data object containing variables (may be NULL)
 ###
 
-parse.formula <- function(form, data) {
+parse.formula <- function(formula, resp.cov = NULL, data) {
   
-  if(missing(data))
-    data = environment(form)
+  allVarsRec <- function(object){
+    if (is.list(object)) {
+      unlist(lapply(object, allVarsRec))
+    }
+    else {
+      all.vars(object)
+    }
+  }
   
-  cl <- match.call()
-  mf <- match.call(expand.dots = FALSE)
-  m <- match(c("form", "data"), names(mf), 0L)
-  mf <- mf[c(1L, m)]
-  mf$drop.unused.levels <- TRUE
-  mf[[1L]] <- quote(stats::model.frame)
-  mf <- eval(mf, parent.frame())
+  if (missing(data) || is.null(data))
+    data = environment(formula)
+
+  names = c(allVarsRec(resp.cov), allVarsRec(formula[[3]]))
+  nrc = switch(is.null(resp.cov)+1, dim(model.matrix(resp.cov))[2]-1,0) 
+  form = eval(parse(text = paste(formula[[2]], "~", paste(names, collapse = "+")))[[1]])
+  
+  mf <- model.frame(form, data)
   mt <- attr(mf, "terms")
   resp <- model.response(mf, "numeric")    	#response from LHS
   if (is.empty.model(mt)) {
-    stop("Formula RHS empty; at least need treatment variable")
+    stop("Formula RHS empty")
   }
   else {
     x <- model.matrix(mt, mf, contrasts)
   }
   
   #extract variables from formula & data
-  trt <- x[,2]				#assume treatment is 1st var on RHS
-  if(dim(x)[2] > 2) {
-    covars <- x[,-c(1,2)]			#variables on RHS, less the intercept, treatment
-  }else{
-    covars = NULL
-  }
   
-  return(list(resp = resp, trt = trt, covars = covars))
+    trt <- x[,nrc+2]				#assume treatment is 1st var on RHS
+    if(dim(x)[2] > 2) {
+      if(!is.null(resp.cov)){
+        covars <- x[,-c(1:(nrc+2))]			#variables on RHS, less the intercept, treatment, response covariates
+        RespX <- x[,(2:(nrc+1))]  		#response-only variables on RHS
+      }else{
+        covars <- x[,-c(1,2)]			#variables on RHS, less the intercept, treatment, response covariates
+        RespX <- NULL
+      } 
+    }else{
+      covars <- NULL
+      RespX <-NULL
+    }
+  
+  return(list(resp = resp, trt = trt, covars = covars, RespX = RespX))
 }
 
+
+
 ###
 #Standardize non-binary variables
 #arguments:
@@ -65,7 +85,7 @@ parse.formula <- function(form, data) {
 
 std.nonbinary <- function(X) {
   #returns standardized values of vector not consisting of only 0s and 1s
-  if(class(X) == "factor")
+  if(class(X) == "factor" | class(X) == "character")
     return(X)
   if(length(unique(X))!=2)
     X = (X - mean(X, na.rm = T))/sd(X, na.rm = T)
