@@ -156,7 +156,17 @@ namespace {
   {
     Scratch& restrict scratch(*static_cast<Scratch*>(scratchPtr));
     
-    double* restrict& temp(probZForU0);
+    ext_leftMultiplyMatrixAndVector(scratch.x, scratch.numObservations, scratch.numPredictors,
+                                    scratch.coefficients, probZForU0);
+    
+    for (size_t i = 0; i < scratch.numObservations; ++i) {
+      double zHatU0 = ext_cumulativeProbabilityOfNormal(probZForU0[i], 0.0, 1.0);
+      double zHatU1 = ext_cumulativeProbabilityOfNormal(probZForU0[i] + zetaZ, 0.0, 1.0);
+      
+      probZForU0[i] = (scratch.z[i] == 1.0 ? zHatU0 : 1.0 - zHatU0);
+      probZForU1[i] = (scratch.z[i] == 1.0 ? zHatU1 : 1.0 - zHatU1);
+    }
+    /* double* restrict& temp(probZForU0);
     ext_leftMultiplyMatrixAndVector(scratch.x, scratch.numObservations, scratch.numPredictors,
                                     scratch.coefficients, temp);
     
@@ -166,7 +176,7 @@ namespace {
       
       probZForU0[i] = (scratch.z[i] == 1.0 ? zHatU0 : 1.0 - zHatU0);
       probZForU1[i] = (scratch.z[i] == 1.0 ? zHatU1 : 1.0 - zHatU1);
-    }
+    } */
   }
   
   void sampleCoefficients(cibart::ProbitTreatmentModel& restrict model, Scratch& restrict scratch, const double* restrict offset)
